@@ -13,12 +13,22 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __POPPLER_FEATURES_H__
 #define __POPPLER_FEATURES_H__
 
 #define POPPLER_HAS_CAIRO 1
+#define POPPLER_WITH_GDK 1
+
+#define POPPLER_MAJOR_VERSION (0)
+#define POPPLER_MINOR_VERSION (12)
+#define POPPLER_MICRO_VERSION (0)
+
+#define POPPLER_CHECK_VERSION(major,minor,micro) \
+  (POPPLER_MAJOR_VERSION > (major) || \
+   (POPPLER_MAJOR_VERSION == (major) && POPPLER_MINOR_VERSION > (minor)) || \
+   (POPPLER_MAJOR_VERSION == (major) && POPPLER_MINOR_VERSION == (minor) && POPPLER_MICRO_VERSION >= (micro)))
 
 #endif /* __POPPLER_FEATURES_H__ */
