@@ -6,6 +6,27 @@
 //
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// Copyright (C) 2005 Martin Kretzschmar <martink@gnome.org>
+// Copyright (C) 2005, 2006 Kristian Høgsberg <krh@redhat.com>
+// Copyright (C) 2005, 2007-2009 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005 Jonathan Blandford <jrb@redhat.com>
+// Copyright (C) 2006, 2007 Jeff Muizelaar <jeff@infidigm.net>
+// Copyright (C) 2006 Takashi Iwai <tiwai@suse.de>
+// Copyright (C) 2006 Ed Catmur <ed@catmur.co.uk>
+// Copyright (C) 2007 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
+// Copyright (C) 2007, 2009 Jonathan Kew <jonathan_kew@sil.org>
+// Copyright (C) 2009 Petr Gajdos <pgajdos@novell.com>
+// Copyright (C) 2009 William Bader <williambader@hotmail.com>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #include <config.h>
 
 #ifdef USE_GCC_PRAGMAS
@@ -23,7 +44,6 @@
 #ifdef WIN32
 #  include <shlobj.h>
 #endif
-#include <fontconfig/fontconfig.h>
 #include "goo/gmem.h"
 #include "goo/GooString.h"
 #include "goo/GooList.h"
@@ -530,12 +550,16 @@ Plugin::~Plugin() {
 // parsing
 //------------------------------------------------------------------------
 
-GlobalParams::GlobalParams() {
+GlobalParams::GlobalParams(const char *customPopplerDataDir)
+  : popplerDataDir(customPopplerDataDir)
+{
   UnicodeMap *map;
   int i;
-  
+
+#ifndef _MSC_VER  
   FcInit();
   FCcfg = FcConfigGetCurrent();
+#endif
 
 #if MULTITHREADED
   gInitMutex(&mutex);
@@ -579,6 +603,7 @@ GlobalParams::GlobalParams() {
   psEmbedTrueType = gTrue;
   psEmbedCIDPostScript = gTrue;
   psEmbedCIDTrueType = gTrue;
+  psSubstFonts = gTrue;
   psPreload = gFalse;
   psOPI = gFalse;
   psASCIIHex = gFalse;
@@ -596,6 +621,7 @@ GlobalParams::GlobalParams() {
   enableFreeType = gTrue;
   antialias = gTrue;
   vectorAntialias = gTrue;
+  forceNoFTAutoHinting = gFalse;
   strokeAdjust = gTrue;
   screenType = screenUnset;
   screenSize = -1;
@@ -653,35 +679,48 @@ GlobalParams::GlobalParams() {
 void GlobalParams::scanEncodingDirs() {
   GDir *dir;
   GDirEntry *entry;
-
-  dir = new GDir(POPPLER_DATADIR "/nameToUnicode", gFalse);
+  const char *dataRoot = popplerDataDir ? popplerDataDir : POPPLER_DATADIR;
+  
+  // allocate buffer large enough to append "/nameToUnicode"
+  size_t bufSize = strlen(dataRoot) + strlen("/nameToUnicode") + 1;
+  char *dataPathBuffer = new char[bufSize];
+  
+  snprintf(dataPathBuffer, bufSize, "%s/nameToUnicode", dataRoot);
+  dir = new GDir(dataPathBuffer, gTrue);
   while (entry = dir->getNextEntry(), entry != NULL) {
-    parseNameToUnicode(entry->getFullPath());
+    if (!entry->isDir()) {
+      parseNameToUnicode(entry->getFullPath());
+    }
     delete entry;
   }
   delete dir;
 
-  dir = new GDir(POPPLER_DATADIR "/cidToUnicode", gFalse);
+  snprintf(dataPathBuffer, bufSize, "%s/cidToUnicode", dataRoot);
+  dir = new GDir(dataPathBuffer, gFalse);
   while (entry = dir->getNextEntry(), entry != NULL) {
     addCIDToUnicode(entry->getName(), entry->getFullPath());
     delete entry;
   }
   delete dir;
 
-  dir = new GDir(POPPLER_DATADIR "/unicodeMap", gFalse);
+  snprintf(dataPathBuffer, bufSize, "%s/unicodeMap", dataRoot);
+  dir = new GDir(dataPathBuffer, gFalse);
   while (entry = dir->getNextEntry(), entry != NULL) {
     addUnicodeMap(entry->getName(), entry->getFullPath());
     delete entry;
   }
   delete dir;
 
-  dir = new GDir(POPPLER_DATADIR "/cMap", gFalse);
+  snprintf(dataPathBuffer, bufSize, "%s/cMap", dataRoot);
+  dir = new GDir(dataPathBuffer, gFalse);
   while (entry = dir->getNextEntry(), entry != NULL) {
     addCMapDir(entry->getName(), entry->getFullPath());
     toUnicodeDirs->append(entry->getFullPath()->copy());
     delete entry;
   }
   delete dir;
+  
+  delete[] dataPathBuffer;
 }
 
 void GlobalParams::parseNameToUnicode(GooString *name) {
@@ -754,10 +793,6 @@ GBool GlobalParams::parseYesNo2(char *token, GBool *flag) {
 }
 
 GlobalParams::~GlobalParams() {
-  GooHashIter *iter;
-  GooString *key;
-  GooList *list;
-
   freeBuiltinFontTables();
 
   delete macRomanReverseMap;
@@ -771,9 +806,7 @@ GlobalParams::~GlobalParams() {
   deleteGooList(toUnicodeDirs, GooString);
   deleteGooHash(displayFonts, DisplayFontParam);
 #ifdef WIN32
-  if (winFontList) {
-    delete winFontList;
-  }
+  delete winFontList;
 #endif
   deleteGooHash(psFonts, PSFontParam);
   deleteGooList(psNamedFonts16, PSFontParam);
@@ -781,8 +814,12 @@ GlobalParams::~GlobalParams() {
   delete textEncoding;
   deleteGooList(fontDirs, GooString);
 
+  GooHashIter *iter;
+  GooString *key;
   cMapDirs->startIter(&iter);
-  while (cMapDirs->getNext(&iter, &key, (void **)&list)) {
+  void *val;
+  while (cMapDirs->getNext(&iter, &key, &val)) {
+    GooList* list = (GooList*)val;
     deleteGooList(list, GooString);
   }
   delete cMapDirs;
@@ -906,7 +943,7 @@ FILE *GlobalParams::findToUnicodeFile(GooString *name) {
   return NULL;
 }
 
-GBool findModifier(const char *name, const char *modifier, const char **start)
+static GBool findModifier(const char *name, const char *modifier, const char **start)
 {
   const char *match;
 
@@ -924,12 +961,13 @@ GBool findModifier(const char *name, const char *modifier, const char **start)
   }
 }
 
+#ifndef _MSC_VER
 static FcPattern *buildFcPattern(GfxFont *font)
 {
-  int weight = FC_WEIGHT_NORMAL,
-      slant = FC_SLANT_ROMAN,
-      width = FC_WIDTH_NORMAL,
-      spacing = FC_PROPORTIONAL;
+  int weight = -1,
+      slant = -1,
+      width = -1,
+      spacing = -1;
   bool deleteFamily = false;
   char *family, *name, *lang, *modifiers;
   const char *start;
@@ -1055,17 +1093,23 @@ static FcPattern *buildFcPattern(GfxFont *font)
   
   p = FcPatternBuild(NULL,
                     FC_FAMILY, FcTypeString, family,
-                    FC_SLANT, FcTypeInteger, slant, 
-                    FC_WEIGHT, FcTypeInteger, weight,
-                    FC_WIDTH, FcTypeInteger, width, 
-                    FC_SPACING, FcTypeInteger, spacing,
                     FC_LANG, FcTypeString, lang,
                     NULL);
+  if (slant != -1) FcPatternAddInteger(p, FC_SLANT, slant);
+  if (weight != -1) FcPatternAddInteger(p, FC_WEIGHT, weight);
+  if (width != -1) FcPatternAddInteger(p, FC_WIDTH, width);
+  if (spacing != -1) FcPatternAddInteger(p, FC_SPACING, spacing);
+
   if (deleteFamily)
     delete[] family;
   return p;
 }
+#endif
 
+/* if you can't or don't want to use Fontconfig, you need to implement
+   this function for your platform. For Windows, it's in GlobalParamsWin.cc
+*/
+#ifndef _MSC_VER
 DisplayFontParam *GlobalParams::getDisplayFont(GfxFont *font) {
   DisplayFontParam *dfp;
   FcPattern *p=0;
@@ -1124,6 +1168,7 @@ fin:
   unlockGlobalParams;
   return dfp;
 }
+#endif
 
 GBool GlobalParams::getPSExpandSmaller() {
   GBool f;
@@ -1237,6 +1282,15 @@ GBool GlobalParams::getPSEmbedCIDTrueType() {
   return e;
 }
 
+GBool GlobalParams::getPSSubstFonts() {
+  GBool e;
+
+  lockGlobalParams;
+  e = psSubstFonts;
+  unlockGlobalParams;
+  return e;
+}
+
 GBool GlobalParams::getPSPreload() {
   GBool preload;
 
@@ -1352,6 +1406,15 @@ GBool GlobalParams::getVectorAntialias() {
   return f;
 }
 
+GBool GlobalParams::getForceNoFTAutoHinting() {
+  GBool f;
+
+  lockGlobalParams;
+  f = forceNoFTAutoHinting;
+  unlockGlobalParams;
+  return f;
+}
+
 GBool GlobalParams::getStrokeAdjust() {
   GBool f;
 
@@ -1473,28 +1536,26 @@ CharCodeToUnicode *GlobalParams::getCIDToUnicode(GooString *collection) {
 }
 
 CharCodeToUnicode *GlobalParams::getUnicodeToUnicode(GooString *fontName) {
-  CharCodeToUnicode *ctu;
-  GooHashIter *iter;
-  GooString *fontPattern, *fileName;
-
   lockGlobalParams;
-  fileName = NULL;
+  GooHashIter *iter;
   unicodeToUnicodes->startIter(&iter);
-  while (unicodeToUnicodes->getNext(&iter, &fontPattern, (void **)&fileName)) {
+  GooString *fileName = NULL;
+  GooString *fontPattern;
+  void *val;
+  while (!fileName && unicodeToUnicodes->getNext(&iter, &fontPattern, &val)) {
     if (strstr(fontName->getCString(), fontPattern->getCString())) {
       unicodeToUnicodes->killIter(&iter);
-      break;
+      fileName = (GooString*)val;
     }
-    fileName = NULL;
   }
+  CharCodeToUnicode *ctu = NULL;
   if (fileName) {
-    if (!(ctu = unicodeToUnicodeCache->getCharCodeToUnicode(fileName))) {
-      if ((ctu = CharCodeToUnicode::parseUnicodeToUnicode(fileName))) {
-	unicodeToUnicodeCache->add(ctu);
-      }
+    ctu = unicodeToUnicodeCache->getCharCodeToUnicode(fileName);
+    if (!ctu) {
+      ctu = CharCodeToUnicode::parseUnicodeToUnicode(fileName);
+      if (ctu)
+         unicodeToUnicodeCache->add(ctu);
     }
-  } else {
-    ctu = NULL;
   }
   unlockGlobalParams;
   return ctu;
@@ -1515,11 +1576,11 @@ UnicodeMap *GlobalParams::getUnicodeMap2(GooString *encodingName) {
   return map;
 }
 
-CMap *GlobalParams::getCMap(GooString *collection, GooString *cMapName) {
+CMap *GlobalParams::getCMap(GooString *collection, GooString *cMapName, Stream *stream) {
   CMap *cMap;
 
   lockCMapCache;
-  cMap = cMapCache->getCMap(collection, cMapName);
+  cMap = cMapCache->getCMap(collection, cMapName, stream);
   unlockCMapCache;
   return cMap;
 }
@@ -1528,6 +1589,25 @@ UnicodeMap *GlobalParams::getTextEncoding() {
   return getUnicodeMap2(textEncoding);
 }
 
+GooList *GlobalParams::getEncodingNames()
+{
+  GooList *result = new GooList;
+  GooHashIter *iter;
+  GooString *key;
+  void *val;
+  residentUnicodeMaps->startIter(&iter);
+  while (residentUnicodeMaps->getNext(&iter, &key, &val)) {
+    result->append(key);
+  }
+  residentUnicodeMaps->killIter(&iter);
+  unicodeMaps->startIter(&iter);
+  while (unicodeMaps->getNext(&iter, &key, &val)) {
+    result->append(key);
+  }
+  unicodeMaps->killIter(&iter);
+  return result;
+}
+
 //------------------------------------------------------------------------
 // functions to set parameters
 //------------------------------------------------------------------------
@@ -1580,6 +1660,12 @@ void GlobalParams::setPSEmbedCIDTrueType(GBool embed) {
   unlockGlobalParams;
 }
 
+void GlobalParams::setPSSubstFonts(GBool substFonts) {
+  lockGlobalParams;
+  psSubstFonts = substFonts;
+  unlockGlobalParams;
+}
+
 void GlobalParams::setPSPreload(GBool preload) {
   lockGlobalParams;
   psPreload = preload;
@@ -1661,6 +1747,64 @@ GBool GlobalParams::setVectorAntialias(char *s) {
   return ok;
 }
 
+GBool GlobalParams::setForceNoFTAutoHinting(char *s) {
+  GBool ok;
+
+  lockGlobalParams;
+  ok = parseYesNo2(s, &forceNoFTAutoHinting);
+  unlockGlobalParams;
+  return ok;
+}
+
+void GlobalParams::setStrokeAdjust(GBool adjust)
+{
+  lockGlobalParams;
+  strokeAdjust = adjust;
+  unlockGlobalParams;
+}
+
+void GlobalParams::setScreenType(ScreenType st)
+{
+  lockGlobalParams;
+  screenType = st;
+  unlockGlobalParams;
+}
+
+void GlobalParams::setScreenSize(int size)
+{
+  lockGlobalParams;
+  screenSize = size;
+  unlockGlobalParams;
+}
+
+void GlobalParams::setScreenDotRadius(int radius)
+{
+  lockGlobalParams;
+  screenDotRadius = radius;
+  unlockGlobalParams;
+}
+
+void GlobalParams::setScreenGamma(double gamma)
+{
+  lockGlobalParams;
+  screenGamma = gamma;
+  unlockGlobalParams;
+}
+
+void GlobalParams::setScreenBlackThreshold(double blackThreshold)
+{
+  lockGlobalParams;
+  screenBlackThreshold = blackThreshold;
+  unlockGlobalParams;
+}
+
+void GlobalParams::setScreenWhiteThreshold(double whiteThreshold)
+{
+  lockGlobalParams;
+  screenWhiteThreshold = whiteThreshold;
+  unlockGlobalParams;
+}
+
 void GlobalParams::setMapNumericCharNames(GBool map) {
   lockGlobalParams;
   mapNumericCharNames = map;
