@@ -1,8 +1,8 @@
 /* poppler-document.cc: qt interface to poppler
  * Copyright (C) 2005, Net Integration Technologies, Inc.
  * Copyright (C) 2005, 2008, Brad Hards <bradh@frogmouth.net>
- * Copyright (C) 2005-2008, Albert Astals Cid <aacid@kde.org>
- * Copyright (C) 2006-2008, Pino Toscano <pino@kde.org>
+ * Copyright (C) 2005-2009, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2006-2009, Pino Toscano <pino@kde.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <Stream.h>
 #include <Catalog.h>
 #include <DateInfo.h>
+#include <GfxState.h>
 
 #include <QtCore/QDebug>
 #include <QtCore/QFile>
@@ -36,6 +37,10 @@
 
 #include "poppler-private.h"
 
+#if defined(USE_CMS)
+#include <lcms.h>
+#endif
+
 namespace Poppler {
 
   int DocumentData::count = 0;
@@ -181,7 +186,11 @@ namespace Poppler {
     QList<FontInfo> Document::fonts() const
     {
 	QList<FontInfo> ourList;
-	scanForFonts(numPages(), &ourList);
+	FontIterator it( 0, m_doc );
+	while ( it.hasNext() )
+	{
+		ourList += it.next();
+	}
 	return ourList;
     }
 
@@ -192,18 +201,23 @@ namespace Poppler {
 
     bool Document::scanForFonts( int numPages, QList<FontInfo> *fontList ) const
     {
-	GooList *items = m_doc->m_fontInfoScanner->scan( numPages );
-
-	if ( NULL == items )
-	    return false;
-
-	for ( int i = 0; i < items->getLength(); ++i ) {
-	    fontList->append( FontInfo(FontInfoData((::FontInfo*)items->get(i))) );
+	if ( !m_doc->m_fontInfoIterator )
+		return false;
+	if ( !m_doc->m_fontInfoIterator->hasNext() )
+		return false;
+	while ( m_doc->m_fontInfoIterator->hasNext() && numPages )
+	{
+		(*fontList) += m_doc->m_fontInfoIterator->next();
+		--numPages;
 	}
-	deleteGooList(items, ::FontInfo);
 	return true;
     }
 
+    FontIterator* Document::newFontIterator( int startPage ) const
+    {
+	return new FontIterator( startPage, m_doc );
+    }
+
     QByteArray Document::fontData(const FontInfo &fi) const
     {
 	QByteArray result;
@@ -365,7 +379,15 @@ namespace Poppler {
 
     double Document::pdfVersion() const
     {
-	return m_doc->doc->getPDFVersion();
+	return m_doc->doc->getPDFMajorVersion () + m_doc->doc->getPDFMinorVersion() / 10.0;
+    }
+
+    void Document::getPdfVersion(int *major, int *minor) const
+    {
+	if (major)
+	    *major = m_doc->doc->getPDFMajorVersion();
+	if (minor)
+	    *minor = m_doc->doc->getPDFMinorVersion();
     }
 
     Page *Document::page(const QString &label) const
@@ -403,11 +425,8 @@ namespace Poppler {
 
     LinkDestination *Document::linkDestination( const QString &name )
     {
-        if ( m_doc->getOutputDev() == NULL )
-            return NULL;
-
         GooString * namedDest = QStringToGooString( name );
-        LinkDestinationData ldd(NULL, namedDest, m_doc);
+        LinkDestinationData ldd(NULL, namedDest, m_doc, false);
         LinkDestination *ld = new LinkDestination(ldd);
         delete namedDest;
         return ld;
@@ -418,6 +437,44 @@ namespace Poppler {
         m_doc->setPaperColor(color);
     }
     
+    void Document::setColorDisplayProfile(void* outputProfileA)
+    {
+#if defined(USE_CMS)
+        GfxColorSpace::setDisplayProfile((cmsHPROFILE)outputProfileA);
+#else
+        Q_UNUSED(outputProfileA);
+#endif
+    }
+
+    void Document::setColorDisplayProfileName(const QString &name)
+    {
+#if defined(USE_CMS)
+        GooString *profileName = QStringToGooString( name );
+        GfxColorSpace::setDisplayProfileName(profileName);
+        delete profileName;
+#else
+        Q_UNUSED(name);
+#endif
+    }
+
+    void* Document::colorRgbProfile() const
+    {
+#if defined(USE_CMS)
+        return (void*)GfxColorSpace::getRGBProfile();
+#else
+        return NULL;
+#endif
+    }
+
+    void* Document::colorDisplayProfile() const
+    {
+#if defined(USE_CMS)
+       return (void*)GfxColorSpace::getDisplayProfile();
+#else
+       return NULL;
+#endif
+    }
+
     QColor Document::paperColor() const
     {
     	return m_doc->paperColor;
@@ -550,4 +607,13 @@ namespace Poppler {
         return QDateTime();
     }
 
+    bool isCmsAvailable()
+    {
+#if defined(USE_CMS)
+        return true;
+#else
+        return false;
+#endif
+    }
+
 }
