@@ -1,5 +1,6 @@
 /* poppler-link.h: qt interface to poppler
  * Copyright (C) 2006, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2007-2008, Pino Toscano <pino@kde.org>
  * Adapting code from
  *   Copyright (C) 2004 by Enrico Ros <eros.kde@email.it>
  *
@@ -24,6 +25,7 @@
 #include <QtCore/QString>
 #include <QtCore/QRectF>
 #include <QtCore/QSharedDataPointer>
+#include "poppler-export.h"
 
 namespace Poppler {
 
@@ -33,21 +35,35 @@ class LinkExecutePrivate;
 class LinkBrowsePrivate;
 class LinkActionPrivate;
 class LinkSoundPrivate;
+class LinkJavaScriptPrivate;
 class LinkMoviePrivate;
 class LinkDestinationData;
 class LinkDestinationPrivate;
 class SoundObject;
 
 /**
- * \short Encapsulates data that describes a link destination.
+ * \short A destination.
+ *
+ * The LinkDestination class represent a "destination" (in terms of visual
+ * viewport to be displayed) for \link Poppler::LinkGoto GoTo\endlink links,
+ * and items in the table of contents (TOC) of a document.
  *
  * Coordinates are in 0..1 range
  */
-class LinkDestination
+class POPPLER_QT4_EXPORT LinkDestination
 {
 	public:
+		/**
+		 * The possible kind of "viewport destination".
+		 */
 		enum Kind
 		{
+			/**
+			 * The new viewport is specified in terms of:
+			 * - possibile new left coordinate (see isChangeLeft() )
+			 * - possibile new top coordinate (see isChangeTop() )
+			 * - possibile new zoom level (see isChangeZoom() )
+			 */
 			destXYZ = 1,
 			destFit = 2,
 			destFitH = 3,
@@ -58,8 +74,10 @@ class LinkDestination
 			destFitBV = 8
 		};
 
+		/// \cond PRIVATE
 		LinkDestination(const LinkDestinationData &data);
 		LinkDestination(const QString &description);
+		/// \endcond
 		/**
 		 * Copy constructor.
 		 */
@@ -74,14 +92,45 @@ class LinkDestination
 		 * The kind of destination.
 		 */
 		Kind kind() const;
+		/**
+		 * Which page is the target of this destination.
+		 *
+		 * \note this number is 1-based, so for a 5 pages document the
+		 *       valid page numbers go from 1 to 5 (both included).
+		 */
 		int pageNumber() const;
+		/**
+		 * The new left for the viewport of the target page, in case
+		 * it is specified to be changed (see isChangeLeft() )
+		 */
 		double left() const;
 		double bottom() const;
 		double right() const;
+		/**
+		 * The new top for the viewport of the target page, in case
+		 * it is specified to be changed (see isChangeTop() )
+		 */
 		double top() const;
 		double zoom() const;
+		/**
+		 * Whether the left of the viewport on the target page should
+		 * be changed.
+		 *
+		 * \see left()
+		 */
 		bool isChangeLeft() const;
+		/**
+		 * Whether the top of the viewport on the target page should
+		 * be changed.
+		 *
+		 * \see top()
+		 */
 		bool isChangeTop() const;
+		/**
+		 * Whether the zoom level should be changed.
+		 *
+		 * \see zoom()
+		 */
 		bool isChangeZoom() const;
 
 		/**
@@ -89,6 +138,16 @@ class LinkDestination
 		 */
 		QString toString() const;
 
+		/**
+		 * Return the name of this destination.
+		 *
+		 * \since 0.12
+		 */
+		QString destinationName() const;
+
+		/**
+		 * Assignment operator.
+		 */
 		LinkDestination& operator=(const LinkDestination &other);
 
 	private:
@@ -102,26 +161,28 @@ class LinkDestination
  * kind of links to reimplement the linkType() method and return the type of
  * the link described by the reimplemented class.
  */
-class Link
+class POPPLER_QT4_EXPORT Link
 {
 	public:
+		/// \cond PRIVATE
 		Link( const QRectF &linkArea );
+		/// \endcond
 		
 		/**
 		 * The possible kinds of link.
 		 *
-		 * \internal
 		 * Inherited classes must return an unique identifier
 		 */
 		enum LinkType
 		{
 		    None,     ///< Unknown link
 		    Goto,     ///< A "Go To" link
-		    Execute,
-		    Browse,
-		    Action,
+		    Execute,  ///< A command to be executed
+		    Browse,   ///< An URL to be browsed (eg "http://poppler.freedesktop.org")
+		    Action,   ///< A "standard" action to be executed in the viewer
 		    Sound,    ///< A link representing a sound to be played
-		    Movie
+		    Movie,    ///< An action to be executed on a movie
+		    JavaScript    ///< A JavaScript code to be interpreted \since 0.10
 		};
 
 		/**
@@ -154,8 +215,15 @@ class Link
 };
 
 
-/** Goto: a viewport and maybe a reference to an external filename **/
-class LinkGoto : public Link
+/**
+ * \brief Viewport reaching request.
+ *
+ * With a LinkGoto link, the document requests the specified viewport to be
+ * reached (aka, displayed in a viewer). Furthermore, if a file name is specified,
+ * then the destination refers to that document (and not to the document the
+ * current LinkGoto belongs to).
+ */
+class POPPLER_QT4_EXPORT LinkGoto : public Link
 {
 	public:
 		/**
@@ -177,7 +245,14 @@ class LinkGoto : public Link
 		 */
 		bool isExternal() const;
 		// query for goto parameters
+		/**
+		 * The file name of the document the destination() refers to,
+		 * or an empty string in case it refers to the current document.
+		 */
 		QString fileName() const;
+		/**
+		 * The destination to reach.
+		 */
 		LinkDestination destination() const;
 		LinkType linkType() const;
 
@@ -186,14 +261,24 @@ class LinkGoto : public Link
 		Q_DISABLE_COPY( LinkGoto )
 };
 
-/** Execute: filename and parameters to execute **/
-class LinkExecute : public Link
+/**
+ * \brief Generic execution request.
+ *
+ * The LinkExecute link represent a "file name" execution request. The result
+ * depends on the \link fileName() file name\endlink:
+ * - if it is a document, then it is requested to be open
+ * - otherwise, it represents an executable to be run with the specified parameters
+ */
+class POPPLER_QT4_EXPORT LinkExecute : public Link
 {
 	public:
 		/**
 		 * The file name to be executed
 		 */
 		QString fileName() const;
+		/**
+		 * The parameters for the command.
+		 */
 		QString parameters() const;
 
 		/**
@@ -215,8 +300,15 @@ class LinkExecute : public Link
 		Q_DISABLE_COPY( LinkExecute )
 };
 
-/** Browse: an URL to open, ranging from 'http://' to 'mailto:', etc. **/
-class LinkBrowse : public Link
+/**
+ * \brief An URL to browse.
+ *
+ * The LinkBrowse link holds a URL (eg 'http://poppler.freedesktop.org',
+ * 'mailto:john@some.org', etc) to be open.
+ *
+ * The format of the URL is specified by RFC 2396 (http://www.ietf.org/rfc/rfc2396.txt)
+ */
+class POPPLER_QT4_EXPORT LinkBrowse : public Link
 {
 	public:
 		/**
@@ -242,8 +334,13 @@ class LinkBrowse : public Link
 		Q_DISABLE_COPY( LinkBrowse )
 };	
 
-/** Action: contains an action to perform on document / viewer **/
-class LinkAction : public Link
+/**
+ * \brief "Standard" action request.
+ *
+ * The LinkAction class represents a link that request a "standard" action
+ * to be performed by the viewer on the displayed document.
+ */
+class POPPLER_QT4_EXPORT LinkAction : public Link
 {
 	public:
 		/**
@@ -286,8 +383,12 @@ class LinkAction : public Link
 		Q_DISABLE_COPY( LinkAction )
 };
 
-/** Sound: a sound to be played **/
-class LinkSound : public Link
+/**
+ * Sound: a sound to be played.
+ *
+ * \since 0.6
+ */
+class POPPLER_QT4_EXPORT LinkSound : public Link
 {
 	public:
 		// create a Link_Sound
@@ -307,12 +408,23 @@ class LinkSound : public Link
 		 * - 1: full volume
 		 */
 		double volume() const;
+		/**
+		 * Whether the playback of the sound should be synchronous
+		 * (thus blocking, waiting for the end of the sound playback).
+		 */
 		bool synchronous() const;
 		/**
 		 * Whether the sound should be played continuously (that is,
 		 * started again when it ends)
 		 */
 		bool repeat() const;
+		/**
+		 * Whether the playback of this sound can be mixed with
+		 * playbacks with other sounds of the same document.
+		 *
+		 * \note When false, any other playback must be stopped before
+		 *       playing the sound.
+		 */
 		bool mix() const;
 		/**
 		 * The sound object to be played
@@ -324,8 +436,41 @@ class LinkSound : public Link
 		Q_DISABLE_COPY( LinkSound )
 };
 
+/**
+ * JavaScript: a JavaScript code to be interpreted.
+ *
+ * \since 0.10
+ */
+class POPPLER_QT4_EXPORT LinkJavaScript : public Link
+{
+	public:
+		/**
+		 * Create a new JavaScript link.
+		 *
+		 * \param linkArea the active area of the link
+		 * \param js the JS code to be interpreted
+		 */
+		LinkJavaScript( const QRectF &linkArea, const QString &js );
+		/**
+		 * Destructor.
+		 */
+		virtual ~LinkJavaScript();
+
+		LinkType linkType() const;
+
+		/**
+		 * The JS code
+		 */
+		QString script() const;
+
+	private:
+		Q_DECLARE_PRIVATE( LinkJavaScript )
+		Q_DISABLE_COPY( LinkJavaScript )
+};	
+
+#if 0
 /** Movie: Not yet defined -> think renaming to 'Media' link **/
-class LinkMovie : public Link
+class POPPLER_QT4_EXPORT LinkMovie : public Link
 // TODO this (Movie link)
 {
 	public:
@@ -337,6 +482,7 @@ class LinkMovie : public Link
 		Q_DECLARE_PRIVATE( LinkMovie )
 		Q_DISABLE_COPY( LinkMovie )
 };
+#endif
 
 }
 
