@@ -61,7 +61,9 @@ void PDFDisplay::display()
         Poppler::Page *page = doc->page(m_currentPage);
         if (page) {
             qDebug() << "Displaying page using" << backendString << "backend: " << m_currentPage;
+            QTime t = QTime::currentTime();
             image = page->renderToImage();
+            qDebug() << "Rendering took" << t.msecsTo(QTime::currentTime()) << "msecs";
             qDeleteAll(textRects);
             if (showTextRects)
             {
@@ -124,12 +126,16 @@ void PDFDisplay::keyPressEvent( QKeyEvent *e )
 
 void PDFDisplay::mousePressEvent( QMouseEvent *e )
 {
+  int i = 0;
   foreach(Poppler::TextBox *tb, textRects)
   {
     if (tb->boundingBox().contains(e->pos()))
     {
-      QToolTip::showText(e->globalPos(), tb->text(), this);
+      QString tt = QString("Text: \"%1\"\nIndex in text list: %2").arg(tb->text()).arg(i);
+      QToolTip::showText(e->globalPos(), tt, this);
+      break;
     }
+    ++i;
   }
 }
 
