@@ -7,6 +7,22 @@
 // Changed 1999 by G.Ovtcharov
 //========================================================================
 
+//========================================================================
+//
+// Modified under the Poppler project - http://poppler.freedesktop.org
+//
+// All changes made under the Poppler project to this file are licensed
+// under GPL version 2 or later
+//
+// Copyright (C) 2006, 2007, 2009 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008-2009 Warren Toomey <wkt@tuhs.org>
+// Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
+//
+// To see a description of the changes please see the Changelog file that
+// came with your tarball or type make ChangeLog if you are building from git
+//
+//========================================================================
+
 #ifndef HTMLOUTPUTDEV_H
 #define HTMLOUTPUTDEV_H
 
@@ -228,7 +244,7 @@ public:
   virtual GBool interpretType3Chars() { return gFalse; }
 
   // Does this device need non-text content?
-  virtual GBool needNonText() { return gFalse; }
+  virtual GBool needNonText() { return gTrue; }
 
   //----- initialization and control
 
@@ -264,10 +280,10 @@ public:
   virtual void drawImageMask(GfxState *state, Object *ref, 
 			     Stream *str,
 			     int width, int height, GBool invert,
-			     GBool inlineImg);
+			     GBool interpolate, GBool inlineImg);
   virtual void drawImage(GfxState *state, Object *ref, Stream *str,
-			  int width, int height, GfxImageColorMap *colorMap,
-			 int *maskColors, GBool inlineImg);
+			 int width, int height, GfxImageColorMap *colorMap,
+			 GBool interpolate, int *maskColors, GBool inlineImg);
 
   //new feature    
   virtual int DevType() {return 1234;}
@@ -281,7 +297,7 @@ private:
   // convert encoding into a HTML standard, or encoding->getCString if not
   // recognized
   static char* mapEncodingToHtml(GooString* encoding);
-  void processLink(Link *link);
+  void doProcessLink(Link *link);
   GooString* getLinkDest(Link *link,Catalog *catalog);
   void dumpMetaVars(FILE *);
   void doFrame(int firstPage);
@@ -301,6 +317,7 @@ private:
   int maxPageWidth;
   int maxPageHeight;
   static int imgNum;
+  static GooList *imgList;
   GooString *Docname;
   GooString *docTitle;
   GooList *glMetaVars;
